;; Generated package description from gnuplot.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "gnuplot" "0.12.0.20260628.3" "Major-mode and interactive frontend for gnuplot" '((emacs "29.1") (compat "31")) :commit "6e758c7ad7bb2865357b13e10ba8c4a1d39ba9c6" :maintainer '(("Maxime Tréca" . "maxime@gmail.com") ("Daniel Mendler" . "mail@daniel-mendler.de")) :keywords '("data" "gnuplot" "plotting") :url "https://github.com/emacs-gnuplot/gnuplot")
